==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/re/rfc822.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: rfc822.c,v 1.4 2003/01/02 16:48:47 lwa Exp $";

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_ADDR_LEN 1000

#ifdef DEBUG_RFC822
#define DEBUG(a, x...) fprintf(stderr, "***DEBUG " a "\n" , ##x)
#else
#define DEBUG(x...)
#endif

static char *skip_CFWS(char *addr) {
  DEBUG("skip_CFWS(%s)", addr);
  while(*addr) {
    switch(*addr) {
    case '(': {
      int parents = 1;
      while(*addr && parents > 0) {
	switch(*addr++) {
	case '\\':
	  if (*addr)
	    addr ++;
	  break;
	case '(':
	  parents++;
	  break;
	case ')':
	  parents--;
	  break;
	case '"':
	  while(*addr && *addr != '"') {
	    if (*addr == '\\' && addr[1])
	      addr+=2;
	    else
	      addr+=1;
	  }
	  addr ++;
	  break;
	}
      }
    }
    case '\t': /* FALLTHROUGH */
    case ' ':
    case '\r':
    case '\n':
      addr++;
      break;
    default:
      return addr;
    }
  }
  return addr;
}

char * rfc822_getaddr(char **list) {
  char ret[MAX_ADDR_LEN+1];
  char *xret = ret;
  char *addr = skip_CFWS(*list); 
  char *start ;
  int angle = 0;
  while(*addr == ',')
    addr = skip_CFWS(addr+1);
  start = addr;
  while(1) {
    addr = skip_CFWS(addr);
    if (*addr == 0)
      break;
    if (*addr == '<') {
      angle = 1;
      xret = ret;
      addr ++;
      continue;
    }
    if (*addr == '>' && angle) {
      addr ++;
      angle = 0;
      break;
    }
    if (*addr == ',' && !angle) {
      addr++;
      break;
    }
    if (*addr == ':') { /* skip begining of group */
      addr++;
      xret = ret;
      continue;
    }
    if (*addr == ';') { /* end of group */
      addr++;
      DEBUG("test=%d", xret == ret && !angle);
      if (xret == ret && !angle) {
	start = addr; /* empty group, reset */
	continue;  
      }
      else 
	break;
    }

    if (*addr == '"') {
      while(*++addr && *addr != '"') {
	if (xret >= ret + MAX_ADDR_LEN)
	  return NULL;
	if (*addr == '\\') {
	  if (*++addr)
	    *xret ++ = *addr;
	} else 
	  *xret ++ = *addr;
      }
      if (*addr)
	addr ++;
      continue;
    }
    if (xret >= ret + MAX_ADDR_LEN)
      return NULL;
    if (*addr == '\\') {
      if (*++addr)
	*xret ++ = *addr ++;
    } else 
      *xret ++ = *addr ++;
  }
  *xret = 0;
  while(*addr == ',')
    addr = skip_CFWS(addr+1);
  *list = addr;
  if (addr == start)
    return NULL;
  DEBUG("return <%s>", ret);
  return strdup(ret);
}

#ifdef TEST_RFC822
int main (int argc, char **argv) {
  char *list = argv[1];
  char *addr;
  while (addr = rfc822_getaddr(&list)) {
    printf("<%s>\n", addr);
    free(addr);
  }
  return 0;
}
#endif

==> lmtpd-0.9.24.tar.gz: lmtpd-0.9.24/re/rfc822.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: rfc822.c,v 1.5 2004/10/26 09:59:21 lwa Exp $";



#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_ADDR_LEN 1000

#ifdef DEBUG_RFC822
#define DEBUG(a, x...) fprintf(stderr, "***DEBUG " a "\n" , ##x)
#else
#define DEBUG(x...)
#endif

static char *skip_CFWS(char *addr) {
  DEBUG("skip_CFWS(%s)", addr);
  while(*addr) {
    switch(*addr) {
    case '(': {
      int parents = 1;
      while(*addr && parents > 0) {
	switch(*addr++) {
	case '\\':
	  if (*addr)
	    addr ++;
	  break;
	case '(':
	  parents++;
	  break;
	case ')':
	  parents--;
	  break;
	case '"':
	  while(*addr && *addr != '"') {
	    if (*addr == '\\' && addr[1])
	      addr+=2;
	    else
	      addr+=1;
	  }
	  addr ++;
	  break;
	}
      }
    }
    case '\t': /* FALLTHROUGH */
    case ' ':
    case '\r':
    case '\n':
      addr++;
      break;
    default:
      return addr;
    }
  }
  return addr;
}

char * rfc822_getaddr(char **list) {
  char ret[MAX_ADDR_LEN+1];
  char *xret = ret;
  char *addr = skip_CFWS(*list); 
  char *start ;
  int angle = 0;
  while(*addr == ',')
    addr = skip_CFWS(addr+1);
  start = addr;
  while(1) {
    addr = skip_CFWS(addr);
    if (*addr == 0)
      break;
    if (*addr == '<') {
      angle = 1;
      xret = ret;
      addr ++;
      continue;
    }
    if (*addr == '>' && angle) {
      addr ++;
      angle = 0;
      break;
    }
    if (*addr == ',' && !angle) {
      addr++;
      break;
    }
    if (*addr == ':') { /* skip begining of group */
      addr++;
      xret = ret;
      continue;
    }
    if (*addr == ';') { /* end of group */
      addr++;
      DEBUG("test=%d", xret == ret && !angle);
      if (xret == ret && !angle) {
	start = addr; /* empty group, reset */
	continue;  
      }
      else 
	break;
    }

    if (*addr == '"') {
      while(*++addr && *addr != '"') {
	if (xret >= ret + MAX_ADDR_LEN)
	  return NULL;
	if (*addr == '\\') {
	  if (*++addr)
	    *xret ++ = *addr;
	} else 
	  *xret ++ = *addr;
      }
      if (*addr)
	addr ++;
      continue;
    }
    if (xret >= ret + MAX_ADDR_LEN)
      return NULL;
    if (*addr == '\\') {
      if (*++addr)
	*xret ++ = *addr ++;
    } else 
      *xret ++ = *addr ++;
  }
  *xret = 0;
  while(*addr == ',')
    addr = skip_CFWS(addr+1);
  *list = addr;
  if (addr == start)
    return NULL;
  DEBUG("return <%s>", ret);
  return strdup(ret);
}

#ifdef TEST_RFC822
int main (int argc, char **argv) {
  char *list = argv[1];
  char *addr;
  while (addr = rfc822_getaddr(&list)) {
    printf("<%s>\n", addr);
    free(addr);
  }
  return 0;
}
#endif
